debug(7 | 34);

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.net.*;
import java.io.*;

sub bytes {
	local('$temp');
	$temp = $1;

	if ($temp < 1024) {
		return round($temp, 1) . "b";
	}

	$temp /= 1024.0;
	if ($temp < 1024) {
		return round($temp, 1) . "kb";
	}

	$temp /= 1024.0;
	return round($temp, 1) . "mb";
}

sub get {
	local('$url $yc $handle $read $data $buffer $report $finger');
	$url = [new URL: $1];
	$yc = [$url openConnection];
	[$yc setUseCaches: 0];
	[$yc setInstanceFollowRedirects: 1];
	[$yc connect];

	$finger = [updater.Main checkServerTrusted: [$yc getServerCertificates]];
	if ($finger ne "1f42950980a1598f576770580abb4c79d4b8a42f") {
		println("[-] SSL certificate verification for www.cobaltstrike.com failed.\n    Observed SHA1 fingerprint: $finger $+ \n    Expected SHA1 fingerprint: 1f42950980a1598f576770580abb4c79d4b8a42f\n\n    Is someone performing SSL intercept? Don't trust this update unless\n    you know why this certificate fingerprint doesn't match.");
	}

	$handle = [SleepUtils getIOHandle: [$yc getInputStream], $null];

	if ($2) {
		# download the file in chunks and report each successful chunk.
		$buffer = allocate(1024 * 1024 * 20);
		$read = 0;
		$report = ticks();
		while $data (readb($handle, 1024 * 512)) {
			$read += strlen($data);
			writeb($buffer, $data);

			if ((ticks() - $report) > 5000) {
				println("Downloaded " . bytes($read));
				$report = ticks();
			}

			sleep(rand() * 1000);
		}
		println("Read " . bytes($read));
		closef($buffer);
		$data = readb($buffer, -1);
		closef($buffer);
		return $data;
	}
	else {
		$data = readb($handle, -1);
		closef($handle);
		return $data;
	}
}

inline finish {
	return;
}

sub ask {
	local('$answer');
	print("(yes/no): ");
	while (1) {
		$answer = readln();
		if (lc($answer) eq "yes") {
			return 1;
		}
		else if (lc($answer) eq "no") {
			return $null;
		}
	}
}

sub error {
	println('[-] ' . $1);
}

sub report {
	println('[+] ' . $1);
}

sub updateProcess {
	local('$check $digest $handle $latest $license $url $data $file');

	# retrieve the license information
	$file = getFileProper(systemProperties()["user.home"], ".cobaltstrike.license");
	if (!-exists $file) {
		error("I can not find your license key, please enter it now:");
		while (1) {
			$license = readln();
			$license = ["$license" trim];
			if ($license eq "") {
				report("Please enter another license key:");
			}
			else {
				$check = [get("https://www.cobaltstrike.com/update?license= $+ $license", $null) trim];
				sleep(1000);
				if ($check !ismatch 'Latest: ([a-zA-Z0-9]{32}) (https://.*)') {
					error($check);
					report("Please enter another license key:");
				}
				else {
					break;
				}
			}
		}
		
		# write out the license.
		$handle = openf("> $+ $file");
		println($handle, $license);
		closef($handle);
	}
	else {
		$handle = openf($file);
		$license = readln($handle);
		closef($handle);
	}

	# check that our binary is ok
	if (!-exists "cobaltstrike.jar") {
		error("I can not find cobaltstrike.jar. Aborting.");
		finish();
	}
	else if (!-canread "cobaltstrike.jar") {
		error("I can not read cobaltstrike.jar (permissions issue?). Aborting.");
		finish();
	}
	
	if (!-canwrite "cobaltstrike.jar") {
		error("I can not write to cobaltstrike.jar. Make sure it's not running. Aborting.");
		finish();
	}

	# digest our binary
	report("Calculate MD5: cobaltstrike.jar");

	$handle = openf("cobaltstrike.jar");
	$digest = digest($handle, "MD5");
	skip($handle, lof("cobaltstrike.jar"));
	$digest = unpack("H*", digest($digest))[0];

	report("MD5: cobaltstrike.jar - $digest");

	# check our license and get the latest hash for cobalt strike
	report("Checking for latest version");
	$check = [get("https://www.cobaltstrike.com/update?license= $+ $license", $null) trim];

	if ($check ismatch 'Latest: ([a-zA-Z0-9]{32}) (https://.*)') {
		($latest, $url) = matched();
		if ($latest eq $digest) {
			report("Your installation of Cobalt Strike is up to date!");
		}
		else {
			report("Downloading the latest version of Cobalt Strike");

			# update cobalt strike...
			$data = get($url, 1);
			report("Downloaded Cobalt Strike. Now verifying the data");
			$check = unpack("H*", digest($data, "MD5"))[0];
			if ($check eq $latest) {
				report("Download verified. Updating cobaltstrike.jar");
				$handle = openf(">cobaltstrike.jar");
				writeb($handle, $data);
				closef($handle);
				report("SUCCESS! Updated cobaltstrike.jar");

				# update the releasenotes.txt file
				$data = get("https://www.cobaltstrike.com/releasenotes.txt", $null);
				$handle = openf(">releasenotes.txt");
				writeb($handle, $data);
				closef($handle);

				report("Downloaded and updated releasenotes.txt. Be sure to read this file!");
			}
			else {
				error("I downloaded cobaltstrike.jar, but the hash doesn't match the expected value. Try again. If this problem continues, please contact support@strategiccyber.com.");
			}
		}
	}
	else {
		error($check);
	}

	report("Done!");
	finish();
}

try {
	updateProcess();
}
catch $exception {
	error("Something in the update process failed:\n $+ $exception");
	printAll(getStackTrace());
}
